/***
  You should have received a copy of the GNU Lesser General Public License
  along with PulseAudio; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.
***/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>

#include <pulse/xmalloc.h>

#include <pulsecore/macro.h>
#include <pulsecore/sink-input.h>
#include <pulsecore/module.h>
#include <pulsecore/modargs.h>
#include <pulsecore/namereg.h>
#include <pulsecore/log.h>

#include "module-deepin-workaround-symdef.h"

PA_MODULE_AUTHOR("snyh");
PA_MODULE_DESCRIPTION("workaound: no audio when headphone is plugged in after login.");
PA_MODULE_VERSION(PACKAGE_VERSION);
PA_MODULE_LOAD_ONCE(true);

static void find_sink_and_source(pa_card *card, pa_device_port *port, pa_sink **si, pa_source **so) {
    pa_sink *sink = NULL;
    pa_source *source = NULL;
    uint32_t state;

    switch (port->direction) {
        case PA_DIRECTION_OUTPUT:
            PA_IDXSET_FOREACH(sink, card->sinks, state)
                if (port == pa_hashmap_get(sink->ports, port->name))
                    break;
            break;

        case PA_DIRECTION_INPUT:
            PA_IDXSET_FOREACH(source, card->sources, state)
                if (port == pa_hashmap_get(source->ports, port->name))
                    break;
            break;
    }

    *si = sink;
    *so = source;
}

static pa_device_port* find_best_port(pa_hashmap *ports) {
    void *state;
    pa_device_port* port, *result = NULL;

    PA_HASHMAP_FOREACH(port, ports, state) {
        if (port->available == PA_AVAILABLE_YES) {
            if (result == NULL) {
                result = port;
                continue;
            }
            if (port->priority > result->priority) {
                result = port;
            }
        }
    }
    if (result) {
        return result;
    }

    PA_HASHMAP_FOREACH(port, ports, state) {
        if (port->available == PA_AVAILABLE_UNKNOWN) {
            if (result == NULL) {
                result = port;
                continue;
            }
            if (port->priority > result->priority) {
                result = port;
            }
        }
    }
    if (result) {
        return result;
    }

    PA_HASHMAP_FOREACH(port, ports, state) {
        if (port->available == PA_AVAILABLE_NO) {
            if (result == NULL) {
                result = port;
                continue;
            }
            if (port->priority > result->priority) {
                result = port;
            }
        }
    }
    if (result) {
        return result;
    }

    return NULL;
}

static void fix_invalid_active_port(pa_core* core, pa_sink *sink) {
    pa_device_port* port = find_best_port(sink->ports);
    if (port) {
        pa_sink_set_port(sink, port->name, true);
        pa_log_info("Fix the sink of \"%s\": active_port form \"%s\" to \"%s\"\n", sink->name, sink->active_port->name, port->name);
    }
}

int pa__init(pa_module*m) {
    pa_card *card;
    uint32_t state;
    PA_IDXSET_FOREACH(card, m->core->cards, state) {
        pa_device_port *port;
        void *state2;

        PA_HASHMAP_FOREACH(port, card->ports, state2) {
            pa_sink *sink = NULL;
            pa_source *source = NULL;
            find_sink_and_source(card, port, &sink, &source);
            if (sink && sink->active_port && (sink->active_port->available == PA_AVAILABLE_UNKNOWN)) {
                fix_invalid_active_port(m->core, sink);
            }
        }
    }
    return 0;
}

void pa__done(pa_module*m) {
}
